/*****************************************************************************
 *
 * Copyright (C) 2002 Polytechnic University of Valencia.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors: Carlos Calafate, <calafate@disca.upv.es>
 *
 *****************************************************************************/


#include "PICA/log.h"

int PICAopenFile(FDesc * file, char * name, int read_write, int flags) {

	if (file == NULL) {
		P_ERROR("PICAopenFile: file descriptor points to NULL", 90101);
		return 0;
	}

	if (name == NULL) {
		P_ERROR("PICAopenFile: file name points to NULL", 90102);
		return 0;
	}

	
	if (read_write == READF) {
		
		if ((*file = CreateFile(name,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL)) == NULL) {
			P_ERROR("PICAopenFile: error opening file", 90103);	
			return 0;		
		}
		return 1;

	} else if (read_write == WRITEF){
		
		if (flags == CREATE_CLEAN) {
			if ((*file = CreateFile(name,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL)) == NULL) {
				P_ERROR("PICAopenFile: error opening file", 90104);	
				return 0;
			}
			return 1;

		} else if (flags == APPEND) {
			if ((*file = CreateFile(name,GENERIC_WRITE,0,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL)) == NULL) {
				P_ERROR("PICAopenFile: error opening file", 90105);	
				return 0;
			}
			return 1;

		} else {
			P_ERROR("PICAopenFile: unknown value for flags", 90106);	
			return 0;
		}

	} else if (read_write == READF_WRITEF) {

		if (flags == CREATE_CLEAN) {
			if ((*file = CreateFile(name,GENERIC_READ|GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL)) == NULL) {
					P_ERROR("PICAopenFile: error opening file", 90107);
					return 0;
			}
			return 1;

			} else if (flags == APPEND) {
			if ((*file = CreateFile(name,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL)) == NULL) {
				P_ERROR("PICAopenFile: error opening file", 90108);	
				return 0;
			}
				return 1;

			} else {
			    P_ERROR("PICAopenFile: unknown value for flags", 90109);
				return 0;
			}

	} else {
	    P_ERROR("PICAopenFile: unknown read/write mode", 90110);	
		return 0;
	}

	return 1;
}


int PICAwriteToFile(FDesc file, void * data, unsigned int datasize) {
	DWORD escr, total_written, remain;
	byte * curr_pos;
	
	if (data == NULL) {
		P_ERROR("PICAwriteToFile: data points to NULL", 90201);
		return 0;
	}
  
	if (datasize < 1) {
	    P_ERROR("PICAwriteToFile: invalid data size", 90202);
		return 0;
	}


	total_written = 0;	
	curr_pos = (byte *) data;
	remain = datasize;

	do {		
		if (SetFilePointer (file, 0, NULL, FILE_END)== INVALID_SET_FILE_POINTER)
		{
			 P_ERROR("PICAwriteToFile: error moving at the file's end", 90203);	
			return 0;
		}
		if (WriteFile(file,(void *) curr_pos,remain,&escr,NULL) == 0) {
		    P_ERROR("PICAwriteToFile: error writing file", 90204);	
			return 0;		
		}
		total_written += escr;
		curr_pos += escr;
		remain -=escr;
	} while (total_written != datasize);


	return 1;
}


int PICAreadFile(FDesc file, void * buf, int buffersize, int * datasize) {
	
	if (buf == NULL) {
		P_ERROR("PICAreadFile: buffer points to NULL", 90301);
		return 0;
	}

	if (datasize == NULL) {
		P_ERROR("PICAreadFile: datasize points to NULL", 90302);
		return 0;
	}
 
	if (buffersize < 1) {
		P_ERROR("PICAreadFile: invalid array size", 90303);
		return 0;
	}
 

	if (ReadFile(file,buf,buffersize, datasize,NULL) == 0) {
		P_ERROR("PICAreadFile: error reading file", 90304);	
		return 0;
	}		

	return 1;
}


int PICAcloseFile(FDesc file) {

	if(!CloseHandle(file)) {
	    P_ERROR("PICAcloseFile: error closing file", 90401);
		return 0;
	}

	return 1;
}


